SystemOrganization addCategory: #'Coral-Utilities'!

Object subclass: #CoralInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Utilities'!

!CoralInstaller commentStamp: 'DamienPollet 7/28/2011 16:04' prior: 0!
A CoralInstaller is responsible to install accessory files to integrate Coral with the shell and environment of the host system.
!

!CoralInstaller class methodsFor: 'file creation' stamp: 'DamienPollet 8/13/2011 00:47'!
createFileNamed: aFileName withContents: aString
	| file |
	file := FSFilesystem disk referenceTo: aFileName.
	file writeStreamDo: [ :io |
		io truncate; nextPutAll: aString withUnixLineEndings ].
	^ file! !

!CoralInstaller class methodsFor: 'file creation' stamp: 'DamienPollet 8/7/2011 00:38'!
createWrapperNamed: aString withContents: contentString
	| script |
	script := self createFileNamed: aString withContents: contentString.
	OSProcess command: 'chmod u+x "' , script pathString , '"'! !

!CoralInstaller class methodsFor: 'script generation' stamp: 'OlivierAuverlot 11/24/2010 14:19'!
generateAllExamples
	"self generateAllExamples"
	(CoralScriptExamples class allMethodsInCategory: #'script examples') do: [ :selector | 
		self generateScript: selector 	
	]

! !

!CoralInstaller class methodsFor: 'script generation' stamp: 'DamienPollet 9/29/2011 16:50'!
generateCoralScript
	"self generateCoralScript"

	self createWrapperNamed: 'coral.sh' withContents:
'#!! /bin/bash
vm=''', Smalltalk vmFullPath, '''
image=''', (Smalltalk getSystemAttribute: 1), '''

imageArgs=""
vmArgs=""
if [[ "$1" == "-i" ]]; then
    shift
    restore_tty="stty `stty -g`"
    stty -icanon -echo
fi
if [[ "$1" == "-d" ]]; then
    shift
    imageArgs+="-debug"
else
    vmArgs+="-headless"
fi

export LINES=`tput lines`
export COLUMNS=`tput cols`

tmp=`mktemp -d -t coral`
echo $tmp
mkfifo "$tmp"/stderr
tail -n +2 "$tmp"/stderr >&2 &

function cleanup() {
    rm -r "$tmp"
    eval $restore_tty
    echo "ok, bye..."
}
trap "cleanup ; trap 2 ; kill -2 $$" 1 2 3 13 15

if [[ $# > 0 ]]; then
   script="$1"
   shift
   "$vm" $vmArgs "$image" "`pwd`/$script" $imageArgs --coral "$@" 2>"$tmp"/stderr
else
   "$vm" $vmArgs "$image" "" 2>"$tmp"/stderr
fi

cleanup
'! !

!CoralInstaller class methodsFor: 'script generation' stamp: 'AdrienBarreau 10/13/2010 14:11'!
generateScript: aSelector
	self createFileNamed: aSelector asString, '.cst' withContents: (CoralScriptExamples perform: aSelector)! !

Object subclass: #CoralREADME
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Utilities'!

!CoralREADME commentStamp: '<historical>' prior: 0!
This project lives here:

http://www.squeaksource.com/Coral/

To load Coral, simply evaluate the following in a fresh image:

ScriptLoader new installer ss project: 'Coral'; install: 'LoadCoral'.

For more information (workspace documentation on class side), just evaluate:

self initialize
!

!CoralREADME class methodsFor: 'fileExample' stamp: 'StephaneDucasse 2/14/2010 11:29'!
creatingAFile

	"str := FSUnixFilesystem new writeStreamOn: 'giffle'.
str nextPutAll: 'zzzzzzzzzz'.
str close"

"(FSDiskFilesystem current childrenAt: FSDiskFilesystem current
workingDirectory) do:
   [:each | each printOn: Transcript cr ].
Transcript flush.

Speaking of current dir...
FSDiskFilesystem current workingDirectory isWorkingDirectory"

"FSDiskFilesystem current workingDirectory"! !

!CoralREADME class methodsFor: 'comment' stamp: 'StephaneDucasse 2/12/2010 16:19'!
faq
	^ '     '! !

!CoralREADME class methodsFor: 'private' stamp: 'StephaneDucasse 9/3/2009 17:45'!
openWorkspaceLabeled: label withContents: contents
	| info |
	info := Workspace new.
	info contents: contents.
	info openLabel: label! !

!CoralREADME class methodsFor: 'initialization' stamp: 'DamienPollet 10/6/2011 17:07'!
openWorkspaces
	
	self openWorkspaceLabeled: 'READ ME' withContents: self readMe.
	self openWorkspaceLabeled: 'TO DO' withContents: self todo.
	self openWorkspaceLabeled: 'FAQ' withContents: self faq.! !

!CoralREADME class methodsFor: 'comment' stamp: 'OlivierAuverlot 11/26/2010 15:27'!
readMe

	^ '"INSTRUCTIONS

TO REVISED.... 

[We assume you are on a Mac -- you will need to adapt these instructions for other platforms ...]"

"1) Save this image as coral.image"

SmalltalkImage current saveAs: ''coral.image''.

"2) Generate coral.sh "

CoralInstaller generateCoralScript.
CoralInstaller generateCoralDebugScript.

"3)  make them executable: chmod +x coral.sh coralDebug.sh"

"4) Generate examples implemented in CoralScriptExamples"

CoralInstaller generateAllExamples.

"5) Execute an example"

"./coral.sh hello.cst"

'! !

!CoralREADME class methodsFor: 'comment' stamp: 'StephaneDucasse 12/8/2009 18:36'!
todo
	^ '"TO DO ITEMS
- clean up dead code (current)
- check extensions made in previous package 
- merge to get back the ancestors
- check ScriptInterpreter?? 
	Do we need to create a class for a script
	How do we store a execution component:  [ ] and list of them [ ]
	for now we can only manipulate scripts outside the image. 

- refresh the readme
- make sure we can pass a script
- remove dependencies on fileman

- support for #!! notation (done)
- installer for #!! script runner

- filename or rio for file
- debug mode for scripts (escape to image)
- passing arguments to scripts
- pipes and filters (stdin and stdout)
- redirecting Smalltalk errors to stderr
- evaluating external Unix commands (cf. backtick commands)
- multi-=platform support (installers?)
- environment to develop and debug scripts (extend class browser?)
- demo scripts
- tutorial"'! !

!CoralREADME class methodsFor: 'fileExample' stamp: 'StephaneDucasse 2/15/2010 18:51'!
tutorial

	"str := FSUnixFilesystem new writeStreamOn: 'giffle'.
str nextPutAll: 'zzzzzzzzzz'.
str close

FSDiskFilesystem current workingDirectory allChildren

FSDiskFilesystem current workingDirectory allEntries


FSDiskFilesystem current root -> / 
FSDiskFilesystem current root class -> FSReference

|working cache|
working := FSDiskFilesystem current working.
cache := working / 'package-cache'.
cache entry creationdate
	
	
	(FSDiskFilesystem current childrenAt: FSDiskFilesystem current workingDirectory )
	
	FSPath / 'plonk' / 'leaffile'. 
		/plonk/leaffile
	FSPath * 'plonk' / 'leaffile'. 
		 plonk/leaffile
	
	
	
	
	"! !

Object subclass: #CoralScriptExamples
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Utilities'!

!CoralScriptExamples commentStamp: 'DamienPollet 7/28/2011 16:05' prior: 0!
I'm a library of example Coral scripts.

I could probably be merged into CoralInstaller.!

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'OlivierAuverlot 11/24/2010 14:17'!
scriptCarpet
		"generates a carpet with random colors"

^'[
	Console resetAllAttributes.
	Console eraseScreen.

	(1 to: 80) do: [ :x | 
		(1 to: 20) do: [ :y |
			Console displayAttributes: { ((10 atRandom) + 29) . 7 . ((10 atRandom) + 39) }.
			Console plot: x @ y.
		]
	].
	
	Console resetAllAttributes.

]'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'AdrienBarreau 10/13/2010 14:13'!
scriptCreationOfFolderAndFiles
	"CoralInstaller generateScript: #scriptCreationOfFolderAndFiles"

	^ '"This script takes one parameter: a path. It will create a folder named CoralGenerated at the path.
If this folder exists, it is deleted first. In CoralGenerated, a few folders and a file are created."

[
	| fs folder |

	fs := FSDiskFilesystem currentClass createDefault.
	folder := CoralScriptWriting getAllArgumentsWithoutMinusButScriptName at:1 ifAbsent: [ String empty ].
	
	folder isEmpty 
		ifTrue: [ Console<<(''Use: '', CoralScriptWriting getScriptName, '' path'') ]
		ifFalse: [ 
			| ref |

			ref := fs referenceTo: folder.
			ref exists
				ifFalse: [ Console printError: ''The path should exist'' ]
				ifTrue: [
					| stream baseRef |
					baseRef := (ref / ''CoralGenerated'').
					baseRef exists ifTrue: [ baseRef deleteAll ].
					baseRef createDirectory.
					((ref / ''CoralGenerated'') / ''a'') createDirectory.
					((ref / ''CoralGenerated'') / ''b'') createDirectory.
					(((ref / ''CoralGenerated'') / ''b'') / ''c'') createDirectory.
					stream := ((((ref / ''CoralGenerated'') / ''b'') / ''c'') / ''coral.txt'') writeStream.
					stream nextPutAll: ''FileGenerated with Coral''.
					stream close.
					Console<<(CoralScriptWriting getScriptName, '' done.'')
				]
		]
]	
'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'AdrienBarreau 5/10/2011 17:55'!
scriptFewThingsWithOSProcess
	^
'"This script does nothing useful ... but it shows a couple of things you can use while scripting"

[
	| process |
	
	process := OSProcess thisOSProcess.
	
	Console << ''Let''''s take a look at many points of OSProcess!!'' << String lf.
	Console << ''------------------------------------------------------------'' << String lf << String lf.
	
	Console << ''Environment variables:'' << String lf.
	Console << ''------'' << String lf.
	process environment associationsDo: [ :each | Console << each key << '' = '' << each value << String lf ].
	
	Console << String lf << ''You can run shell commands. Here is "ls -la *.cst":'' << String lf.
	Console << ''------'' << String lf.
	process waitForCommand: ''ls -la *.cst''.
	
	Console << String lf << ''You can change the working directory.'' << String lf.
	Console << ''------'' << String lf..
	Console << ''pwd: ''.
	process waitForCommand: ''pwd''.
	Console << ''Changing working directory for /'' << String lf.
	process chDir: ''/''.
	Console << ''pwd: ''.
	process waitForCommand: ''pwd''.
	Console << ''ls:'' << String lf.
	process waitForCommand: ''ls''.

	Console << String lf << CoralScriptWriting getScriptName << '' done.''
]
'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'AdrienBarreau 10/13/2010 16:36'!
scriptForScriptTestCoralScriptWriting
	"CoralInstaller generateScript: #scriptForScriptTestCoralScriptWriting"

	^ '[
	| fs os |
	
	fs := FSDiskFilesystem currentClass createDefault.
	os := ThisOSProcess thisOSProcess.
	
	os chDir: (fs referenceTo: ''.'') asAbsolute asString.
	CoralInstaller generateScript: #scriptTestCoralScriptWriting.
	
	os waitForCommand: ''./coral.sh scriptTestCoralScriptWriting.cst arg1 arg2 -minus1 arg3 -minus2 -minus3 arg4 arg5''.
	
	(fs referenceTo: ''./scriptTestCoralScriptWriting.cst'') delete.
	
	Console << (CoralScriptWriting getScriptName, '' done.'')
]	
'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'OlivierAuverlot 11/24/2010 10:41'!
scriptHappyFace

  ^'[
	Console resetAllAttributes.
	Console displayAttributes: #(31 7 42).
	Console eraseScreen.

	Console locate: 10@5; print: ''  ''.
	Console locate: 10@6; print: '' O''.
	Console locate: 30@5; print: ''  ''.
	Console locate: 30@6; print: '' O''.

	Console plot: 20@10.
	Console plot: 20@11.

	Console plot: 14@16.
	Console plot: 25@16.

	Console locate: 15@17; print: ''          ''.
	
	Console locate: 1@20.
	Console resetAllAttributes.
]'
! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'AdrienBarreau 5/10/2011 18:11'!
scriptHello

  ^'[
	Console print: ''Coral is working'', String lf.
	Console print: ''Yahoo!!''
]'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'AdrienBarreau 10/13/2010 14:14'!
scriptPrintArguments
	"CoralInstaller generateScript: #scriptPrintArguments"

	^ '"This script takes * arguments and print them with 3 arguments-using methods"
	
[
	Console printNl:''getScriptName: '', CoralScriptWriting getScriptName.

	Console printNl: ''getAllArgumentsWithoutMinus:''.
	CoralScriptWriting getAllArgumentsWithoutMinus do: [ :each | Console printNl: each ].

	Console printNl: ''getAllArgumentsWithoutMinusButScriptName:''.
	CoralScriptWriting getAllArgumentsWithoutMinusButScriptName do: [ :each | Console printNl: each ]
]	
'! !

!CoralScriptExamples class methodsFor: 'script examples' stamp: 'AdrienBarreau 10/13/2010 14:15'!
scriptSynchronizeSecondFolderWithFirst
	"CoralInstaller generateScript: #scriptSynchronizeSecondFolderWithFirst"

	^ '"This script takes two paths as parameters. They should be two folders. This script will synchronize the
second folder with the first.
NB: Can be improve, this example just add the files which does not exist in the folder 2"

[ Object < #TempCoralClass ]

TempCoralClass class>>synchronizeFolder1: folder1 folder2: folder2
[
	| entries1 baseNames2 filesToCreate |
	
	entries1 := folder1 children.
	baseNames2 := folder2 children collect: [ :each | each basename ].
	filesToCreate := entries1 reject: [ :each |  baseNames2 includes: each basename ].
	
	filesToCreate do: [ :each | 
		each isFile 
			ifTrue: [ 
				| readStream |
				readStream := each readStream.
				(folder2 / (each basename)) writeStream nextPutAll: (readStream contents); close.
				readStream close
			]
			ifFalse: [ (folder2 / (each basename)) createDirectory. self synchronizeFolder1: each folder2: (folder2 / (each basename)) ]
	].
]

[
	| args fs folder1 folder2 |
	
	args := CoralScriptWriting getAllArguments.
	(args size = 3) ifFalse: [ Console<<(''Use: '', args first, '' path1 path2''). ^ self ].
	
	fs := FSDiskFilesystem currentClass createDefault.
	folder1 := fs referenceTo: (args at:2).
	folder2 := fs referenceTo: (args at:3).
	
	(folder1 exists & folder2 exists) ifFalse: [ Console printError: ''Error: The two folders should exist.''. ^ self ].
	
	TempCoralClass synchronizeFolder1: folder1 folder2: folder2.
	Console<< (args first, '' done.'').
]	
'! !

!CoralScriptExamples class methodsFor: 'testing scripts' stamp: 'AdrienBarreau 10/13/2010 16:39'!
scriptTestCoralScriptWriting
	"Don't generate this script, but use scriptForScriptTestCoralScriptWriting"

	^ '[ TestCase < #TempCoralClass ]

TempCoralClass>>testGetAllArguments
[
	| args |
	
	args := CoralScriptWriting getAllArguments.
	
	self assert: (args size = 4).
	self assert: ((args at: #*) = #(''scriptTestCoralScriptWriting.cst'' arg1 arg2)).
	self assert: ((args at: #minus1) = #(arg3)).
	self assert: ((args at: #minus2) = #()).
	self assert: ((args at: #minus3) = #(arg4 arg5)).
]

TempCoralClass>>testGetAllArgumentsWithoutMinus
[
	| args |
	
	args := CoralScriptWriting getAllArgumentsWithoutMinus.
	
	self assert: (args size = 3).
	self assert: ((args at:1) = ''scriptTestCoralScriptWriting.cst'').
	self assert: ((args at:2) = ''arg1'').
	self assert: ((args at:3) = ''arg2'').
	
	self assert: (args = (CoralScriptWriting getAllArguments at: #*)).
]

TempCoralClass>>testGetAllArgumentsWithoutMinusButScriptName
[
	| args |
	
	args := CoralScriptWriting getAllArgumentsWithoutMinusButScriptName.
	
	self assert: (args size = 2).
	self assert: ((args at:1) = ''arg1'').
	self assert: ((args at:2) = ''arg2'').
	
	self assert: (args = (CoralScriptWriting getAllArguments at: #*) allButFirst).
	self assert: (args = CoralScriptWriting getAllArgumentsWithoutMinus allButFirst).
]

TempCoralClass>>testGetScriptName
[
	| arg |
	
	arg := CoralScriptWriting getScriptName.
	
	self assert: (arg = ''scriptTestCoralScriptWriting.cst'').
	self assert: (arg = (CoralScriptWriting getAllArguments at: #*) first).
	self assert: (arg = CoralScriptWriting getAllArgumentsWithoutMinus first).
]

[
	| flag |
	
	flag := true.
	TempCoralClass selectors do: [ :each |
		[ TempCoralClass debug: each ] on: TestFailure do: [ Console printNl: ''Test failure: '', each. flag := false ]
	].

	flag ifTrue: [ Console printNl: ''All tests were successfully passed!!''  ]
]'
! !
